@@ -34,6 +34,9 @@ struct swaylock_colors {
 	uint32_t caps_lock_bs_highlight;
 	uint32_t caps_lock_key_highlight;
 	uint32_t separator;
+	uint32_t layout_background;
+	uint32_t layout_border;
+	uint32_t layout_text;
 	struct swaylock_colorset inside;
 	struct swaylock_colorset line;
 	struct swaylock_colorset ring;
@@ -50,6 +53,8 @@ struct swaylock_args {
 	bool show_indicator;
 	bool show_caps_lock_text;
 	bool show_caps_lock_indicator;
+	bool show_keyboard_layout;
+	bool hide_keyboard_layout;
 	bool show_failed_attempts;
 	bool daemonize;
 };
@@ -65,6 +70,7 @@ struct swaylock_state {
 	struct loop_timer *clear_password_timer;  // clears the password buffer
 	struct wl_display *display;
 	struct wl_compositor *compositor;
+	struct wl_subcompositor *subcompositor;
 	struct zwlr_layer_shell_v1 *layer_shell;
 	struct zwlr_input_inhibit_manager_v1 *input_inhibit_manager;
 	struct wl_shm *shm;
@@ -86,8 +92,11 @@ struct swaylock_surface {
 	uint32_t output_global_name;
 	struct zxdg_output_v1 *xdg_output;
 	struct wl_surface *surface;
+	struct wl_surface *child; // surface made into subsurface
+	struct wl_subsurface *subsurface;
 	struct zwlr_layer_surface_v1 *layer_surface;
 	struct pool_buffer buffers[2];
+	struct pool_buffer indicator_buffers[2];
 	struct pool_buffer *current_buffer;
 	bool frame_pending, dirty;
 	uint32_t width, height;
@@ -107,6 +116,7 @@ struct swaylock_image {
 
 void swaylock_handle_key(struct swaylock_state *state,
 		xkb_keysym_t keysym, uint32_t codepoint);
+void render_frame_background(struct swaylock_surface *surface);
 void render_frame(struct swaylock_surface *surface);
 void render_frames(struct swaylock_state *state);
 void damage_surface(struct swaylock_surface *surface);
