@@ -126,6 +126,12 @@ static void create_layer_surface(struct swaylock_surface *surface) {
 	surface->surface = wl_compositor_create_surface(state->compositor);
 	assert(surface->surface);
 
+	surface->child = wl_compositor_create_surface(state->compositor);
+	assert(surface->child);
+	surface->subsurface = wl_subcompositor_get_subsurface(state->subcompositor, surface->child, surface->surface);
+	assert(surface->subsurface);
+	wl_subsurface_set_sync(surface->subsurface);
+
 	surface->layer_surface = zwlr_layer_shell_v1_get_layer_surface(
 			state->layer_shell, surface->surface, surface->output,
 			ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY, "lockscreen");
@@ -163,6 +169,7 @@ static void layer_surface_configure(void *data,
 	surface->width = width;
 	surface->height = height;
 	zwlr_layer_surface_v1_ack_configure(layer_surface, serial);
+	render_frame_background(surface);
 	render_frame(surface);
 }
 
@@ -297,6 +304,9 @@ static void handle_global(void *data, struct wl_registry *registry,
 	if (strcmp(interface, wl_compositor_interface.name) == 0) {
 		state->compositor = wl_registry_bind(registry, name,
 				&wl_compositor_interface, 3);
+	} else if (strcmp(interface, wl_subcompositor_interface.name) == 0) {
+		state->subcompositor = wl_registry_bind(registry, name,
+				&wl_subcompositor_interface, 1);
 	} else if (strcmp(interface, wl_shm_interface.name) == 0) {
 		state->shm = wl_registry_bind(registry, name,
 				&wl_shm_interface, 1);
@@ -364,6 +374,23 @@ static cairo_surface_t *select_image(struct swaylock_state *state,
 	return default_image;
 }
 
+static char *join_args(char **argv, int argc) {
+	assert(argc > 0);
+	int len = 0, i;
+	for (i = 0; i < argc; ++i) {
+		len += strlen(argv[i]) + 1;
+	}
+	char *res = malloc(len);
+	len = 0;
+	for (i = 0; i < argc; ++i) {
+		strcpy(res + len, argv[i]);
+		len += strlen(argv[i]);
+		res[len++] = ' ';
+	}
+	res[len - 1] = '\0';
+	return res;
+}
+
 static void load_image(char *arg, struct swaylock_state *state) {
 	// [[<output>]:]<path>
 	struct swaylock_image *image = calloc(1, sizeof(struct swaylock_image));
@@ -397,11 +424,19 @@ static void load_image(char *arg, struct swaylock_state *state) {
 		}
 	}
 
-	// Bash doesn't replace the ~ with $HOME if the output name is supplied
+	// The shell will not expand ~ to the value of $HOME when an output name is
+	// given. Also, any image paths given in the config file need to have shell
+	// expansions performed
 	wordexp_t p;
+	while (strstr(image->path, "  ")) {
+		image->path = realloc(image->path, strlen(image->path) + 2);
+		char *ptr = strstr(image->path, "  ") + 1;
+		memmove(ptr + 1, ptr, strlen(ptr) + 1);
+		*ptr = '\\';
+	}
 	if (wordexp(image->path, &p, 0) == 0) {
 		free(image->path);
-		image->path = strdup(p.we_wordv[0]);
+		image->path = join_args(p.we_wordv, p.we_wordc);
 		wordfree(&p);
 	}
 
@@ -423,6 +458,9 @@ static void set_default_colors(struct swaylock_colors *colors) {
 	colors->caps_lock_bs_highlight = 0xDB3300FF;
 	colors->caps_lock_key_highlight = 0x33DB00FF;
 	colors->separator = 0x000000FF;
+	colors->layout_background = 0x000000C0;
+	colors->layout_border = 0x00000000;
+	colors->layout_text = 0xFFFFFFFF;
 	colors->inside = (struct swaylock_colorset){
 		.input = 0x000000C0,
 		.cleared = 0xE5A445C0,
@@ -474,6 +512,9 @@ static int parse_options(int argc, char **argv, struct swaylock_state *state,
 		LO_INSIDE_VER_COLOR,
 		LO_INSIDE_WRONG_COLOR,
 		LO_KEY_HL_COLOR,
+		LO_LAYOUT_TXT_COLOR,
+		LO_LAYOUT_BG_COLOR,
+		LO_LAYOUT_BORDER_COLOR,
 		LO_LINE_COLOR,
 		LO_LINE_CLEAR_COLOR,
 		LO_LINE_CAPS_LOCK_COLOR,
@@ -508,6 +549,8 @@ static int parse_options(int argc, char **argv, struct swaylock_state *state,
 		{"scaling", required_argument, NULL, 's'},
 		{"tiling", no_argument, NULL, 't'},
 		{"no-unlock-indicator", no_argument, NULL, 'u'},
+		{"show-keyboard-layout", no_argument, NULL, 'k'},
+		{"hide-keyboard-layout", no_argument, NULL, 'K'},
 		{"show-failed-attempts", no_argument, NULL, 'F'},
 		{"version", no_argument, NULL, 'v'},
 		{"bs-hl-color", required_argument, NULL, LO_BS_HL_COLOR},
@@ -522,6 +565,9 @@ static int parse_options(int argc, char **argv, struct swaylock_state *state,
 		{"inside-ver-color", required_argument, NULL, LO_INSIDE_VER_COLOR},
 		{"inside-wrong-color", required_argument, NULL, LO_INSIDE_WRONG_COLOR},
 		{"key-hl-color", required_argument, NULL, LO_KEY_HL_COLOR},
+		{"layout-bg-color", required_argument, NULL, LO_LAYOUT_BG_COLOR},
+		{"layout-border-color", required_argument, NULL, LO_LAYOUT_BORDER_COLOR},
+		{"layout-text-color", required_argument, NULL, LO_LAYOUT_TXT_COLOR},
 		{"line-color", required_argument, NULL, LO_LINE_COLOR},
 		{"line-clear-color", required_argument, NULL, LO_LINE_CLEAR_COLOR},
 		{"line-caps-lock-color", required_argument, NULL, LO_LINE_CAPS_LOCK_COLOR},
@@ -552,12 +598,18 @@ static int parse_options(int argc, char **argv, struct swaylock_state *state,
 			"Enable debugging output.\n"
 		"  -e, --ignore-empty-password      "
 			"When an empty password is provided, do not validate it.\n"
+		"  -F, --show-failed-attempts       "
+			"Show current count of failed authentication attempts.\n"
 		"  -f, --daemonize                  "
 			"Detach from the controlling terminal after locking.\n"
 		"  -h, --help                       "
 			"Show help message and quit.\n"
 		"  -i, --image [[<output>]:]<path>  "
 			"Display the given image.\n"
+		"  -k, --show-keyboard-layout       "
+			"Display the current xkb layout while typing.\n"
+		"  -K, --hide-keyboard-layout       "
+			"Hide the current xkb layout while typing.\n"
 		"  -L, --disable-caps-lock-text     "
 			"Disable the Caps Lock text.\n"
 		"  -l, --indicator-caps-lock        "
@@ -568,8 +620,6 @@ static int parse_options(int argc, char **argv, struct swaylock_state *state,
 			"Same as --scaling=tile.\n"
 		"  -u, --no-unlock-indicator        "
 			"Disable the unlock indicator.\n"
-		"  -F, --show-failed-attempts       "
-			"Show current count of failed authentication attempts.\n"
 		"  -v, --version                    "
 			"Show the version number and quit.\n"
 		"  --bs-hl-color <color>            "
@@ -599,6 +649,12 @@ static int parse_options(int argc, char **argv, struct swaylock_state *state,
 			"Sets the color of the inside of the indicator when invalid.\n"
 		"  --key-hl-color <color>           "
 			"Sets the color of the key press highlight segments.\n"
+		"  --layout-bg-color <color>        "
+			"Sets the background color of the box containing the layout text.\n"
+		"  --layout-border-color <color>    "
+			"Sets the color of the border of the box containing the layout text.\n"
+		"  --layout-text-color <color>      "
+			"Sets the color of the layout text.\n"
 		"  --line-color <color>             "
 			"Sets the color of the line between the inside and ring.\n"
 		"  --line-clear-color <color>       "
@@ -647,7 +703,7 @@ static int parse_options(int argc, char **argv, struct swaylock_state *state,
 	optind = 1;
 	while (1) {
 		int opt_idx = 0;
-		c = getopt_long(argc, argv, "c:deFfhi:Llnrs:tuvC:", long_options,
+		c = getopt_long(argc, argv, "c:deFfhi:kKLlnrs:tuvC:", long_options,
 				&opt_idx);
 		if (c == -1) {
 			break;
@@ -671,6 +727,11 @@ static int parse_options(int argc, char **argv, struct swaylock_state *state,
 				state->args.ignore_empty = true;
 			}
 			break;
+		case 'F':
+			if (state) {
+				state->args.show_failed_attempts = true;
+			}
+			break;
 		case 'f':
 			if (state) {
 				state->args.daemonize = true;
@@ -681,6 +742,16 @@ static int parse_options(int argc, char **argv, struct swaylock_state *state,
 				load_image(optarg, state);
 			}
 			break;
+		case 'k':
+			if (state) {
+				state->args.show_keyboard_layout = true;
+			}
+			break;
+		case 'K':
+			if (state) {
+				state->args.hide_keyboard_layout = true;
+			}
+			break;
 		case 'L':
 			if (state) {
 				state->args.show_caps_lock_text = false;
@@ -719,11 +790,6 @@ static int parse_options(int argc, char **argv, struct swaylock_state *state,
 				state->args.show_indicator = false;
 			}
 			break;
-		case 'F':
-			if (state) {
-				state->args.show_failed_attempts = true;
-			}
-			break;
 		case 'v':
 			fprintf(stdout, "swaylock version " SWAYLOCK_VERSION "\n");
 			exit(EXIT_SUCCESS);
@@ -789,6 +855,21 @@ static int parse_options(int argc, char **argv, struct swaylock_state *state,
 				state->args.colors.key_highlight = parse_color(optarg);
 			}
 			break;
+		case LO_LAYOUT_BG_COLOR:
+			if (state) {
+				state->args.colors.layout_background = parse_color(optarg);
+			}
+			break;
+		case LO_LAYOUT_BORDER_COLOR:
+			if (state) {
+				state->args.colors.layout_border = parse_color(optarg);
+			}
+			break;
+		case LO_LAYOUT_TXT_COLOR:
+			if (state) {
+				state->args.colors.layout_text = parse_color(optarg);
+			}
+			break;
 		case LO_LINE_COLOR:
 			if (state) {
 				state->args.colors.line.input = parse_color(optarg);
@@ -989,6 +1070,8 @@ int main(int argc, char **argv) {
 		.show_indicator = true,
 		.show_caps_lock_indicator = false,
 		.show_caps_lock_text = true,
+		.show_keyboard_layout = false,
+		.hide_keyboard_layout = false,
 		.show_failed_attempts = false
 	};
 	wl_list_init(&state.images);
@@ -1059,12 +1142,6 @@ int main(int argc, char **argv) {
 		return 1;
 	}
 
-	if (wl_list_empty(&state.surfaces)) {
-		free(state.args.font);
-		swaylock_log(LOG_ERROR, "Exiting - no outputs to show on.");
-		return 0;
-	}
-
 	zwlr_input_inhibit_manager_v1_get_inhibitor(state.input_inhibit_manager);
 	if (wl_display_roundtrip(state.display) == -1) {
 		free(state.args.font);
